 /*

********************************************************************************
**      ATI Mach 64 chipset driver
********************************************************************************
**
    Copyright (C)       1998    Isaac Connor
                                Mathew Wilkins

********************************************************************************
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
********************************************************************************


*/
#include	<kgi/system.h>
#include	<kgi/kgi.h>
#include	<IBM/vga.h>


#ifndef _MACH64_IO_C__
#define _MACH64_IO_C__

#include	"ATI/mach64_io.h"
#include	"ATI/mach64.h"



#define eieio() /* Enforce In-order Execution of I/O */
#if 0
inline void Mach64_set_PLL( int offset, kgiu8 value, struct cardInfo *card ) 
{
    /* write addr byte */
    mach64_out8(card->Registers[ Clock_Cntl ] + 1, (offset << 2) | PLL_WR_EN, card);
    eieio();
    /* write the register value */
    mach64_out8(card->Registers[ Clock_Cntl ] + 2, value, card);
    eieio();
    mach64_out8(card->Registers[ Clock_Cntl ] + 1, (offset << 2) & ~PLL_WR_EN, card);

}
#endif

inline	int	mach64_out32( int reg, kgiu32 value ,struct kgi_display* dpy)
{
	GET_PRIVATE;
	CHECK_REGISTER
	if (reg == Config_Cntl && card->Using_MMIO == 2)
	{
		io_out32(value, card->SIO_Base + card->Registers[reg]);
		return 1;
	}

	switch (card->Using_MMIO)
	{
	   case 2: mem_out32(value, (kgiu32*)  ( card->MMIO_Base + 
			card->Registers[reg])); break;
	   case 1: io_out32(value, card->SIO_Base + 
			card->Registers[reg]);break;
           case 0: io_out32(value, card->Registers[reg]);	
	}
	return	1;
}

inline	int	mach64_out8( int reg, int offset, 
				kgiu8 value, struct kgi_display* dpy ) {
	GET_PRIVATE;
	CHECK_REGISTER
	CHECK_OFFSET
		
	switch (card->Using_MMIO)
	{
	   case 2: mem_out8(value, (kgiu32*)  ( card->MMIO_Base + 
			card->Registers[reg]) + offset); break;
	   case 1: io_out8(value, card->SIO_Base + 
			card->Registers[reg] + offset );break;
           case 0: io_out8(value, card->Registers[reg] + offset);	
	}
	return	1;
}

/*inline	int	mach64_out16( int reg, kgiu16 value )
{

	CHECK_REGISTER

	return	1;
}*/



inline	kgiu32	mach64_in32( int reg , struct kgi_display* dpy) {
    GET_PRIVATE;
    CHECK_REGISTER

    if (reg == Config_Cntl && card->Using_MMIO == 2)
        return io_in32(card->SIO_Base + card->Registers[reg]);

    switch (card->Using_MMIO)
    {
	case 2: return mem_in32((kgiu32*) (card->MMIO_Base +
			       			card->Registers[reg]));
	case 1:	return io_in32(card->SIO_Base + card->Registers[reg]);
        case 0:	return io_in32(card->Registers[reg]);	
    }
    return 0;
}
inline	kgiu8	mach64_in8( int reg, int offset, struct kgi_display* dpy ) {
    GET_PRIVATE;
    CHECK_REGISTER
    CHECK_OFFSET
    switch (card->Using_MMIO)
    {
	case 2: return mem_in8((kgiu8*) (card->MMIO_Base +
			       			card->Registers[reg] + offset));
	case 1:	return io_in8(card->SIO_Base + card->Registers[reg] + offset);
        case 0:	return io_in8(card->Registers[reg] + offset);	
    }
    return	io_in8( reg + offset );

}
/*
inline	kgiu16	mach64_in16( int reg, int offset ) {

    CHECK_REGISTER
    CHECK_OFFSET

    return	io_in16( reg + offset );

}

*/


void	DAC_out8( struct kgi_display *dpy, kgiu8 val, kgiu8 reg )
{
		if( reg & ~0x3 )
		{
			DEBUG1("DAC register select lines truncated: %x to %x",
				reg, reg &= 0x3);
		}

		io_out8( val, vga_dac_ports[ reg & 0x03 ]);
}


kgiu8	DAC_in8( struct kgi_display *dpy, kgiu8 reg )
{
		if( reg & ~0x03 )
		{
			DEBUG1("DAC register select lines truncated: %x to %x",
				reg, reg &= 0x03);
		}

		return	io_in8( vga_dac_ports[ reg & 0x03 ] );

}


void	CLK_out8( struct kgi_display *dpy, kgiu8 val, kgiu8 reg ) {
		if( reg != 0 ) 
			INTERNAL_ERROR;

		/* a textmodeism... */
		switch( val ) {
			case 0:
				val	= MISC_25MHZ_CLK;
				break;
			case 1:
				val	= MISC_28MHZ_CLK;
				break;
			default:
				INTERNAL_ERROR;
				return;
		}

		io_out8( (io_in8(MISCr) & 0xf3) | val, MISCw );
}


kgiu8	CLK_in8( struct kgi_display *dpy, kgiu8 reg ) {
    if( reg != 0 ) {
        INTERNAL_ERROR;
    }
    return	( io_in8(MISCr) & 0x04 ) >> 2;
}
#endif
